﻿using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.InputSystem;

[RequireComponent(typeof(CharacterController))]
[RequireComponent(typeof(PlayerInput))]
[RequireComponent(typeof(PlayerStatus))]
[RequireComponent(typeof(MobAttack))]
public class PlayerController : SingletonMonoBehaviourInSceneBase<PlayerController>
{
    private static readonly int MoveSpeed = Animator.StringToHash("MoveSpeed");
    [SerializeField] private Animator animator; // Animatorのキャッシュ
    [SerializeField] private float moveSpeed = 3; // 移動速度
    [SerializeField] private float jumpPower = 3; // ジャンプ力
    [SerializeField] private ThrowAxe throwAxePrefab; // 投げオノPrefab
    private MobAttack _attack; // Attackコンポーネントのキャッシュ
    private CharacterController _characterController; // CharacterControllerのキャッシュ
    private InputAction _fire; // Fireアクションのキャッシュ
    private bool _isReadyToThrow; // 投擲スタンバイ中かどうか
    private InputAction _jump; // Jumpアクションのキャッシュ
    private InputAction _move; // Moveアクションのキャッシュ
    private Vector3 _moveVelocity; // キャラの移動速度情報
    private PlayerStatus _status; // PlayerStatusコンポーネントのキャッシュ
    private InputAction _toggleThrowAxe; // ToggleThrowAxeアクションのキャッシュ
    private Transform _transform; // Transformのキャッシュ

    public bool IsReadyToThrow
    {
        get => _isReadyToThrow;
        set
        {
            _isReadyToThrow = value;
            Menu.Instance.IsThrowAxeActive = _isReadyToThrow;
        }
    }

    private void Start()
    {
        _characterController = GetComponent<CharacterController>(); // 毎フレームアクセスするので、負荷を下げるためにキャッシュしておく
        _transform = transform; // Transformもキャッシュすると少しだけ負荷が下がる

        var input = GetComponent<PlayerInput>();
        // PlayerInputの「Default Map」で指定されているアクションマップを有効化
        input.currentActionMap.Enable();
        // アクションマップからアクションを取得するにはFindAction()を使う
        _move = input.currentActionMap.FindAction("Move");
        _fire = input.currentActionMap.FindAction("Fire");
        _jump = input.currentActionMap.FindAction("Jump");
        _toggleThrowAxe = input.currentActionMap.FindAction("ToggleThrowAxe");

        _status = GetComponent<PlayerStatus>();
        _attack = GetComponent<MobAttack>();
    }

    private void Update()
    {
        if (!EventSystem.current.IsPointerOverGameObject() && _fire.WasReleasedThisFrame())
        {
            // Fireアクションが実行された場合の処理
            if (IsReadyToThrow)
            {
                // 投擲スタンバイ中であれば、投擲
                if (OwnedItemsData.Instance.GetItem(Item.ItemType.ThrowAxe).Number <= 0)
                {
                    Debug.Log("投げオノを持ってないよ〜");
                }
                else
                {
                    if (Camera.main != null && Mouse.current != null)
                    {
                        var mousePointerRay = Camera.main.ScreenPointToRay(Mouse.current.position.value);
                        if (Physics.Raycast(mousePointerRay, out var hit))
                        {
                            var throwAxe = Instantiate(throwAxePrefab);
                            var target = hit.collider.name == "Terrain"
                                ? hit.point
                                : hit.collider.transform.position + Vector3.up;
                            throwAxe.Initialize(transform.position + Vector3.up, target, 20f);
                            OwnedItemsData.Instance.Use(Item.ItemType.ThrowAxe);
                            OwnedItemsData.Instance.Save();
                            AudioManager.Instance.Play("throw");
                            if (OwnedItemsData.Instance.GetItem(Item.ItemType.ThrowAxe).Number <= 0)
                                IsReadyToThrow = false;
                        }
                    }
                }
            }
            else
            {
                // 通常攻撃
                _attack.AttackIfPossible();
            }
        }

        if (_toggleThrowAxe.WasReleasedThisFrame())
            // ToggleThrowAxeアクションが実行されたら、投擲スタンバイ/解除
            ToggleReadyToThrow();

        // Debug.Log(IsGrounded ? "地上にいます" : "空中です");

        if (_status.IsMovable) // 移動可能な状態であれば、ユーザ入力を移動に反映する
        {
            // 入力軸による移動処理（慣性を無視しているので、キビキビ動く）
            var moveValue = _move.ReadValue<Vector2>();
            _moveVelocity.x = moveValue.x * moveSpeed;
            _moveVelocity.z = moveValue.y * moveSpeed;

            // 移動方向に向く
            _transform.LookAt(_transform.position + new Vector3(_moveVelocity.x, 0, _moveVelocity.z));
        }
        else
        {
            _moveVelocity.x = 0;
            _moveVelocity.z = 0;
        }

        // Y軸方向は重力による加速が必要なため、別途計算
        if (_characterController.isGrounded)
        {
            if (_jump.WasPressedThisFrame())
            {
                // ジャンプ処理
                Debug.Log("ジャンプ！");
                _moveVelocity.y = jumpPower; // ジャンプの際は上方向にジャンプの初速を加える
            }
        }
        else
        {
            // 重力による加速
            _moveVelocity.y += Physics.gravity.y * Time.deltaTime;
        }

        // オブジェクトを動かす
        _characterController.Move(_moveVelocity * Time.deltaTime);

        // 移動スピードをanimatorに反映
        animator.SetFloat(MoveSpeed, new Vector3(_moveVelocity.x, 0, _moveVelocity.z).magnitude);
    }

    public void ToggleReadyToThrow()
    {
        if (OwnedItemsData.Instance.GetItem(Item.ItemType.ThrowAxe).Number <= 0)
            IsReadyToThrow = false;
        else
            IsReadyToThrow = !IsReadyToThrow;

        Debug.Log("投擲モード" + (IsReadyToThrow ? "ON" : "OFF"));
    }
}